<?php

namespace Axima\Pays\Model;

use Axima\PaymentGate\Utils;
use Axima\Pays\Utils\Database;
use Magento\Framework\App\ObjectManager;
use Magento\Payment\Model\InfoInterface;

class Payment extends \Magento\Payment\Model\Method\AbstractMethod
{

	const METHOD_CODE = 'payscz';
	const CONFIG_PATH = __DIR__ . '/../../../../../var/config_axima_pays.json';

	protected $_code = 'payscz';

	protected $_minOrderTotal = 0;
	protected $_supportedCurrencyCodes = ['CZK', 'USD', 'EUR'];

	public function __construct(\Magento\Framework\Model\Context $context, \Magento\Framework\Registry $registry, \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory, \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory, \Magento\Payment\Helper\Data $paymentData, \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig, \Magento\Payment\Model\Method\Logger $logger, \Magento\Framework\Model\ResourceModel\AbstractResource $resource = NULL, \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = NULL, array $data = [])
	{
		parent::__construct($context, $registry, $extensionFactory, $customAttributeFactory, $paymentData, $scopeConfig, $logger, $resource, $resourceCollection, $data);
	}


	public function canUseForCurrency($currencyCode)
	{
		return in_array($currencyCode, $this->_supportedCurrencyCodes, TRUE);
	}

	public function authorize(InfoInterface $payment, $amount)
	{
		/** @var \Magento\Sales\Model\Order\Payment $payment */
		$order = $payment->getOrder();

		$database = new Database(ObjectManager::getInstance()->get('\Magento\Framework\App\ResourceConnection'));

		$database->insert('axima_pays_payments', array(
			'identifier' => $merchantOrderNumber = 'KRM' . str_pad(mt_rand(), 10, '0', STR_PAD_LEFT),
			'status' => 0,
			'customer' => $order->getBillingAddress()->getFirstname() . ' ' . $order->getBillingAddress()->getLastname() . ', ' . $order->getBillingAddress()->getEmail(),
			'order' => $order->getIncrementId(),
			'amount' => $amount . ' ' . $order->getOrderCurrencyCode(),
			'date_initiated' => date('Y-m-d G:i:s'),
		));

		$link = Utils::getPaymentLink(self::getOption('merchantId', NULL), self::getOption('shopId', NULL), $order->getOrderCurrencyCode(), $amount * 100, $merchantOrderNumber, $order->getCustomerEmail());
		header('X-Payment-Link: ' . $link);
		/*echo '{}';
		exit;*/
	}


	/**
	 * @return array
	 */
	private static function getOptions()
	{
		$config = @file_get_contents(self::CONFIG_PATH);
		if (!$config) {
			$config = array();
		} else {
			$config = json_decode($config, JSON_OBJECT_AS_ARRAY);
		}
		return $config;
	}


	/**
	 * @param string $name
	 * @param mixed $default
	 * @return mixed
	 */
	private static function getOption($name, $default = NULL)
	{
		$options = self::getOptions();
		return isset($options[$name]) ? $options[$name] : $default;
	}

	private static function getHashPassword()
	{
		return self::getOption('hashPassword', '');
	}

}
