<?php
/**
 * @author Tomáš Blatný
 */

namespace Axima\Pays\Setup;


use Axima\Pays\Utils\Database;
use Magento\Framework\App\Area;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\State;
use Magento\Framework\DB\Ddl\Table;
use Magento\Framework\Setup\InstallSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;
use Magento\Framework\Url;


class InstallSchema implements InstallSchemaInterface
{

	public function install(SchemaSetupInterface $setup, ModuleContextInterface $context)
	{
		$installer = $setup;
		$objectManager = ObjectManager::getInstance();

		/** @var Url $urlInterface */
		$urlInterface = $objectManager->get('\Magento\Framework\Url');
		/** @var State $state */
		$state = $objectManager->get('\Magento\Framework\App\State');
		$state->setAreaCode(Area::AREA_FRONTEND);

		$database = new Database($objectManager->get('Magento\Framework\App\ResourceConnection'));
		if (!count($database->select('cms_page', 'WHERE `identifier` = %s', array('payment-ok')))) {
			$database->insert('cms_page', array(
				'title' => 'Payment successful',
				'page_layout' => '1column',
				'identifier' => 'payment-ok',
				'content_heading' => __('Payment was successful.'),
				'content' => '<p>' . __('Your payment was successful, thank you for your purchase.') . '</p>',
			));
		}

		if (!count($database->select('cms_page', 'WHERE `identifier` = %s', array('payment-error')))) {
			$database->insert('cms_page', [
				'title' => 'Payment failed',
				'page_layout' => '1column',
				'identifier' => 'payment-error',
				'content_heading' => __('Payment failed.'),
				'content' => '<p>' . __('Your payment failed, we apologize for trouble.') . '</p>',
			]);
		}

		$rows = $database->select('cms_page', 'WHERE `identifier` = %s OR `identifier` = %s', array('payment-ok', 'payment-error'));
		if (count($rows)) {
			foreach ($rows as $row) {
				$database->insert('cms_page_store', array(
					'page_id' => $row['page_id'],
					'store_id' => 0,
				));
			}
		}

		$data = array(
			'success-url' => $this->getPageUrl($urlInterface->getUrl('payment-ok')),
			'error-url' => $this->getPageUrl($urlInterface->getUrl('payment-error')),
		);
		@file_put_contents(__DIR__ . '/../../../../../var/config_axima_pays.json', json_encode($data));

		$installer->startSetup();

		$tableName = $installer->getTable('axima_pays_payments');
		if (!$installer->getConnection()->isTableExists($tableName)) {
			$table = $installer->getConnection()
				->newTable($tableName)
				->addColumn('id', Table::TYPE_INTEGER, NULL, [
					'identity' => TRUE,
					'unsigned' => TRUE,
					'nullable' => FALSE,
					'primary' => TRUE,
				])
				->addColumn('identifier', Table::TYPE_TEXT, NULL, ['nullable' => FALSE])
				->addColumn('payment_order_id', Table::TYPE_TEXT, NULL, ['nullable' => FALSE])
				->addColumn('status', Table::TYPE_SMALLINT, NULL, ['nullable' => FALSE, 'unsigned' => TRUE])
				->addColumn('customer', Table::TYPE_TEXT, NULL, ['nullable' => FALSE])
				->addColumn('order', Table::TYPE_TEXT, NULL, ['nullable' => FALSE])
				->addColumn('amount', Table::TYPE_TEXT, NULL, ['nullable' => FALSE])
				->addColumn('note', Table::TYPE_TEXT, NULL, ['nullable' => TRUE, 'default' => NULL])
				->addColumn('date_initiated', Table::TYPE_DATETIME, NULL, ['nullable' => TRUE])
				->addColumn('date_paid', Table::TYPE_DATETIME, NULL, ['nullable' => TRUE])
				->setOption('type', 'InnoDB')
				->setOption('charset', 'utf8');
			$installer->getConnection()->createTable($table);
		}

		$installer->endSetup();
	}

	private function getPageUrl($pageUrl)
	{
		if (!$pageUrl) {
			return $pageUrl;
		}
		if (($pos = strpos($pageUrl, '?SID=')) !== FALSE) {
			return substr($pageUrl, 0, $pos);
		}
		return $pageUrl;
	}
}
