<?php
/**
 * @author Tomáš Blatný
 */

namespace Axima\Pays\Utils;


use DateTime;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Adapter\AdapterInterface;
use Zend_Db_Statement_Interface;

class Database
{

	/** @var AdapterInterface */
	private $connection;

	/** @var ResourceConnection */
	private $resourceConnection;


	public function __construct(ResourceConnection $resourceConnection)
	{
		$this->connection = $resourceConnection->getConnection();
		$this->resourceConnection = $resourceConnection;
	}


	/**
	 * @param string $from
	 * @param string|NULL $query
	 * @param array $args
	 * @return array
	 */
	public function select($from, $query = NULL, array $args = array())
	{

		return $this->query('SELECT * FROM `' . $this->getTableName($from) . '` ' . $query, $args);
	}


	public function selectOne($column, $table, $other = NULL, $args = array())
	{
		$query = 'SELECT ' . $column . ' FROM `' . $this->getTableName($table) . '`' . ($other ? (' ' . $other) : '') . ' LIMIT 1';
		if (strpos($query, '%')) {
			$result = $this->connection->fetchOne($this->prepare($query, $args));
		} else {
			$result = $this->connection->fetchOne($query);
		}
		return isset($result[0]) ? $result[0] : NULL;
	}


	public function insert($table, $values, $other = NULL)
	{
		$this->runQuery('INSERT INTO `' . $this->getTableName($table) . '` (`' . implode ('`, `', array_keys($values)) . '`) VALUES (' . $this->getInsertValues($values) . ')' . ($other ? (' ' . $other) : ''), array_values($values));
	}


	public function update($table, $values, $where, $whereArgs)
	{
		$this->runQuery('UPDATE `' . $this->getTableName($table) . '` SET ' . $this->getUpdateValues($values) . ' WHERE ' . $where, array_merge($values, $whereArgs));
	}


	public function delete($table, $where, $whereArgs)
	{
		$this->runQuery('DELETE FROM `' . $this->getTableName($table) . '` WHERE ' . $where, $whereArgs);
	}


	/**
	 * @param string $table
	 * @param array $fields
	 */
	public function createTable($table, array $fields)
	{
		$tableDefinition = array();
		foreach ($fields as $name => $field) {
			$tableDefinition[] = '`' . $name . '` ' . $field;
		}
		$this->runQuery('CREATE TABLE IF NOT EXISTS ' . $this->getTableName($table) . ' (' . implode(",\n", $tableDefinition) . ') COLLATE utf8mb4_unicode_ci');
	}


	/**
	 * @param string $table
	 */
	public function dropTable($table)
	{
		$this->runQuery('DROP TABLE `' . $this->getTableName($table) . '`');
	}


	/**
	 * @param string $query
	 * @param array $args
	 * @return array
	 */
	private function query($query, array $args = array())
	{
		return strpos($query, '%') ? $this->connection->fetchAll($this->prepare($query, $args)) : $this->connection->fetchAll($query);
	}


	/**
	 * @param string $query
	 * @param array $args
	 * @return Zend_Db_Statement_Interface
	 */
	private function runQuery($query, array $args = array())
	{
		return strpos($query, '%') ? $this->connection->query($this->prepare($query, $args)) : $this->connection->query($query);
	}


	private function prepare($query, $args)
	{
		$query = str_replace( ["'%s'", '"%s"'], '%s', $query );
		$query = preg_replace( '|(?<!%)%f|' , '%F', $query );
		$query = preg_replace( '|(?<!%)%s|', "'%s'", $query );
		array_walk($args, function (&$item) {
			if (!is_float($item)) {
				$item = addslashes($item);
			}
		});
		return @vsprintf( $query, $args );
	}


	/**
	 * @param string $table
	 * @return string
	 */
	private function getTableName($table)
	{
		return $this->resourceConnection->getTableName($table);
	}


	private function getInsertValues($values)
	{
		$result = array();
		foreach ($values as &$value) {
			if (is_int($value) || is_bool($value)) {
				$result[] = '%d';
			} else {
				if ($value instanceof DateTime) {
					$value = $value->format('Y-m-d');
				}
				$result[] = '%s';
			}
		}
		return implode(', ', $result);
	}


	private function getUpdateValues($values)
	{
		$result = array();
		foreach ($values as $key => &$value) {
			if (is_int($value) || is_bool($value)) {
				$result[] = '`' . $key . '` = %d';
			} else {
				if ($value instanceof DateTime) {
					$value = $value->format('Y-m-d');
				}
				$result[] = '`' . $key . '` = %s';
			}
		}
		return implode(', ', $result);
	}

}
